@@ -1,10 +1,13 @@
 # coding: utf-8
-__version__ = '1.1.1'
+__version__ = '1.2.0'
 from typing import Any, Dict, List, Optional, Set, Callable, Union
 from collections import UserList
 from typing import List, Optional, Callable
+from warnings import warn
 from . rules import IRule
 from . helper.base import HelperBase
+from .helper import NO_THING
+
 
 # region Custom Errors
 
@@ -26,7 +29,7 @@ class HelperArgs(HelperBase):
         self._field = None
         self._require = False
         self._types = set()
-        self._default = None
+        self._default = NO_THING
         self._rules = []
         self.key = key
         keys = ('field', 'require', 'types', 'default', 'rules')
@@ -41,7 +44,7 @@ class HelperArgs(HelperBase):
             arg['field'] = self.field
         if self.types is not None and len(self.types) > 0:
             arg['types'] = [itm for itm in self.types]
-        if self.default is not None:
+        if self.default is not NO_THING:
             arg['default'] = self.default
         if self.rules is not None and len(self.rules) > 0:
             arg['rules'] = self.rules
@@ -117,8 +120,9 @@ class HelperArgs(HelperBase):
     # endregion Properties
 # endregion class HelperArgs
 
-
 # region class AssignBuilder
+
+
 class AssignBuilder(UserList):
     '''Helper class for building list to use with "KwargsHelper.Assing() method'''
 
@@ -249,6 +253,113 @@ class AssignBuilder(UserList):
 # region Event Args
 
 
+class BeforeAssignAutoEventArgs:
+    def __init__(self, key: str, value: object, field: str, originator: object) -> None:
+        self._key = key
+        self._field_value = value
+        self._cancel = False
+        self._originator = originator
+        self._field_name = field
+
+    # region Properties
+    @property
+    def key(self) -> str:
+        '''
+        Gets the key of the key value pair
+        '''
+        return self._key
+
+    @property
+    def field_name(self) -> str:
+        '''
+        Gets the field that will be assigned representing `key`
+        '''
+        return self._field_name
+
+    @field_name.setter
+    def field_name(self, value: str) -> str:
+        '''
+        Sets the field that will be assigned representing `key`
+        '''
+        self._field_name = str(value).strip()
+
+    @property
+    def field_value(self) -> object:
+        '''
+        Gets the value to be blindly assigned to property
+        '''
+        return self._field_value
+
+    @field_value.setter
+    def field_value(self, value: object) -> None:
+        '''
+        Sets the value to be blindly assigned to property
+        '''
+        self._field_value = value
+
+    @property
+    def originator(self) -> object:
+        '''Gets object that attributes assigned/modified for '''
+        return self._originator
+
+    @property
+    def cancel(self) -> bool:
+        return self._cancel
+
+    @cancel.setter
+    def cancel(self, value: bool) -> None:
+        self._cancel = bool(value)
+    # endregion Properties
+
+
+class AfterAssignAutoEventArgs:
+    def __init__(self, key: str, originator: object) -> None:
+        self._key = key
+        self._field_value: object = None
+        self._originator = originator
+        self._field_name: str = ''
+        self._success = False
+        self._canceled = False
+
+    # region Properties
+    @property
+    def key(self) -> str:
+        '''
+        Gets the key of the key value pair
+        '''
+        return self._key
+
+    @property
+    def field_name(self) -> str:
+        '''
+        Gets the field that will be assigned representing `key`
+        '''
+        return self._field_name
+
+    @property
+    def field_value(self) -> object:
+        '''
+        Gets the value to be blindly assigned to property
+        '''
+        return self._field_value
+
+    @property
+    def originator(self) -> object:
+        '''Gets object that attributes assigned/modified for '''
+        return self._originator
+
+    @property
+    def success(self) -> bool:
+        '''Get assigning of attribue/value succeeded'''
+        return self._success
+
+    @property
+    def canceled(self) -> bool:
+        '''Get if assigment was canceled by before events'''
+        return self._canceled
+    # endregion Properties
+
+
 class BeforeAssignEventArgs:
     def __init__(self, help_args: HelperArgs, originator: object):
         self._helper_args = help_args
@@ -402,9 +513,16 @@ class KwargsHelper(HelperBase):
         @rule_test_before_assign: (optional) Type:bool, sets the `rule_test_before_assign` property
         @assign_true_not_required: (optional): Type:bool, sets the `assign_true_not_required` property
         '''
-        self._callbacks = None
+        if not isinstance(obj_kwargs, dict):
+            raise TypeError(self._get_type_error_method_msg(
+                method_name='__init__', arg=obj_kwargs,
+                arg_name='obj_kwargs', expected_type=dict
+            ))
         self._obj: object = originator
+        self._callbacks = None
         self._kwargs = obj_kwargs
+        self._auto_assigned = False
+        self._keys: set = set(self._kwargs.keys())
         m_name = '__init__'
         key = 'field_prefix'
         if key in kwargs:
@@ -452,10 +570,23 @@ class KwargsHelper(HelperBase):
                 method_name=m_name, arg=self._assign_true_not_required, arg_name=key, raise_error=True)
         else:
             self._assign_true_not_required: bool = True
+
     # endregion init
 
     # region Public Methods
-    def assign(self, key: str, field: Optional[str] = None, require: bool = False, default: Optional[object] = None, types: Optional[List[type]] = None, rules: Optional[List[Callable[[IRule], bool]]] = None) -> bool:
+    def auto_assign(self) -> bool:
+        '''
+        Assigns all of the key, value pairs of `obj_kwargs` passed into constructor to `originator`,
+        unless the event is canceled in `BeforeAssignBlindEventArgs` then key, value pair will be added automacally to `originator`.
+        @return: `True` of all key, value pairs are added; Otherwise, `False`
+        
+        Call back events are supported via `add_handler_before_assign_auto()` and `add_handler_after_assign_auto()` methods.
+        '''
+        if self._is_auto_assign_handlers():
+            return self._auto_assign_with_cb()
+        return self._auto_assign_no_cb()
+
+    def assign(self, key: str, field: Optional[str] = None, require: bool = False, default: Optional[object] = NO_THING, types: Optional[List[type]] = None, rules: Optional[List[Callable[[IRule], bool]]] = None) -> bool:
         '''
         Assigns attribute value to `obj` passed in to constructor. Attributes are created if they do not exist.
         @key: Type:str, the key of the key, value pair that is required or optional in `obj_kwargs` passed into to constructor.
@@ -490,7 +621,7 @@ class KwargsHelper(HelperBase):
         # _args.require = require
         _args.types = set(types)
         _args.rules = rules
-        if default is not None:
+        if default is not NO_THING:
             _args.default = default
         before_args = BeforeAssignEventArgs(_args, self._obj)
 
@@ -514,7 +645,7 @@ class KwargsHelper(HelperBase):
                                 arg_name='helper', arg_type=HelperArgs, raise_error=True)
         d = helper.to_dict()
         return self.assign(**d)
-    
+
     def is_key_existing(self, key: str) -> bool:
         '''
         Gets if the key exist in  kwargs dictionary passed in to the constructor by `obj_kwargs` arg.
@@ -527,6 +658,45 @@ class KwargsHelper(HelperBase):
     # endregion Public Methods
 
     # region private methods
+    def _auto_assign_no_cb(self) -> bool:
+        for k, v in self._kwargs.items():
+            field_name = f"{self._field_prefix}{k}"
+            setattr(self._obj, field_name, v)
+            # self._obj.__dict__[field_name] = v
+            self._remove_key(key=k)
+        return True
+
+    def _auto_assign_with_cb(self) -> bool:
+        '''
+        Assigns all key, value pairs blindly unless interupted by event cancel
+        @return: `True` if all key, values have been added; Otherwise, `False`
+        '''
+        self._auto_assigned == True
+        result = True
+        for k, v in self._kwargs.items():
+            field_name = f"{self._field_prefix}{k}"
+            before_args = BeforeAssignAutoEventArgs(
+                key=k, value=v, field=field_name, originator=self._obj)
+            self._on_before_assign_auto(event_args=before_args)
+            _field = before_args.field_name
+            _value = before_args.field_value
+            after_args = AfterAssignAutoEventArgs(key=k, originator=self._obj)
+            after_args._canceled = before_args.cancel
+            if before_args.cancel == True:
+                if self._cancel_error == True:
+                    raise CancelEventError(
+                        f"{self.__class__.__name__}.assign() canceled in 'BeforeAssignBlindEventArgs'")
+                else:
+                    result = False
+            else:
+                # self._obj.__dict__[_field] = _value
+                setattr(self._obj, _field, _value)
+                after_args._success = True
+                after_args._field_name = _field
+                after_args._field_value = _value
+                self._remove_key(key=k)
+            self._on_after_assign_auto(event_args=after_args)
+        return result
     def _assign(self, args: HelperArgs, before_args: BeforeAssignEventArgs, after_args: AfterAssignEventArgs) -> None:
         result = False
         key = args.key
@@ -543,7 +713,7 @@ class KwargsHelper(HelperBase):
                 result = self._setattr(f"{self._field_prefix}{key}",
                                        value, before_args, after_args, args=args)
         else:
-            if args.default is not None:
+            if args.default is not NO_THING:
                 if args.field:
                     result = self._setattr(args.field, args.default,
                                            before_args, after_args, args=args)
@@ -569,7 +739,7 @@ class KwargsHelper(HelperBase):
         before_args.field_name = field
         before_args.field_value = value
 
-        self._on_before_assign(before_args=before_args)
+        self._on_before_assign(event_args=before_args)
 
         after_args._canceled = before_args.cancel
         if before_args.cancel == True:
@@ -586,6 +756,8 @@ class KwargsHelper(HelperBase):
             if result == False:
                 return result
         setattr(self._obj, _field, _value)
+        # self._obj.__dict__[_field] = _value
+        self._remove_key(args.key)
         if self._rule_test_early == False:
             result = self._validate_rules(
                 args=args, field=_field, value=_value, after_args=after_args)
@@ -609,6 +781,27 @@ class KwargsHelper(HelperBase):
         after_args._rules_passed = True
         return True
 
+    def _remove_key(self, key: str) -> bool:
+        '''
+        Removes a key from the internal keys.
+        @return: `True` if key was removed; Otherwise, `False`
+        '''
+        if key in self._keys:
+            self._keys.remove(key)
+            return True
+        return False
+
+    def _is_auto_assign_handlers(self) -> bool:
+        '''
+        Gets if any handler are set for Auto Assign
+        '''
+        if self._callbacks is None:
+            return False
+        if 'on_before_assign_auto' in self._callbacks:
+            return True
+        if 'on_after_assign_auto' in self._callbacks:
+            return True
+        return False
     # endregion private methods
 
     # region callback funcs
@@ -628,13 +821,19 @@ class KwargsHelper(HelperBase):
                 callback(self, eventArgs)
 
     # endregion callback funcs
-
+    
     # region raise events
-    def _on_before_assign(self, before_args: BeforeAssignEventArgs):
-        self._trigger("on_before_assign", before_args)
+    def _on_before_assign(self, event_args: BeforeAssignEventArgs):
+        self._trigger("on_before_assign", event_args)
 
     def _on_after_assign(self, event_args: AfterAssignEventArgs):
         self._trigger("on_after_assign", event_args)
+
+    def _on_before_assign_auto(self, event_args: BeforeAssignAutoEventArgs):
+        self._trigger("on_before_assign_auto", event_args)
+
+    def _on_after_assign_auto(self, event_args: AfterAssignAutoEventArgs):
+        self._trigger("on_after_assign_auto", event_args)
     # endregion raise events
 
      # region Handlers
@@ -643,6 +842,12 @@ class KwargsHelper(HelperBase):
 
     def add_handler_after_assign(self, callback: Callable[['KwargsHelper', AfterAssignEventArgs], None]):
         self._on("on_after_assign", callback)
+
+    def add_handler_before_assign_auto(self, callback: Callable[['KwargsHelper', BeforeAssignAutoEventArgs], None]):
+        self._on("on_before_assign_auto", callback)
+
+    def add_handler_after_assign_auto(self, callback: Callable[['KwargsHelper', AfterAssignAutoEventArgs], None]):
+        self._on("on_after_assign_auto", callback)
     # endregion
 
     # region Properties
@@ -807,11 +1012,19 @@ class KwargsHelper(HelperBase):
         '''Gets the kwargs dictionary passed in to the constructor by `obj_kwargs` arg'''
         return self._kwargs
 
+    @property
+    def unused_keys(self) -> Set[str]:
+        '''
+        Gets any unused keys passed into constructor via `obj_kwargs`
+
+        This would be a set of keys that were never used passed into the constructor.
+        '''
+        return self._keys
     # endregion Properties
 
 # endregion class KwargsHelper
 
-# region class KwArgs
+# region class KwArg
 
 
 class KwArg(HelperBase):
@@ -823,7 +1036,7 @@ class KwArg(HelperBase):
          '_get_type_error_prop_msg', '_isinstance_prop',
          '_prop_error', '_is_prop_str', '_is_prop_bool', '_is_prop_int',
          '_isinstance_method', '_is_arg_str', '_is_arg_bool',
-         '_arg_type_error', '_get_name_type_obj'])
+         '_arg_type_error', '_get_name_type_obj', 'kw_unused_keys', 'kw_auto_assign', 'kw_assign', 'kw_assign_helper'])
 
     def __init__(self, **kwargs):
         '''
@@ -854,7 +1067,24 @@ class KwArg(HelperBase):
             originator=self, obj_kwargs={**kwargs})
         self._kw_args_helper_class_instance.field_prefix = ''
 
-    def assign(self, key: str, field: Optional[str] = None, require: bool = False, default: Optional[object] = None, types: Optional[List[type]] = None, rules: Optional[List[Callable[[IRule], bool]]] = None) -> bool:
+    def kw_auto_assign(self):
+        '''
+        Assigns all of the key, value pairs of `obj_kwargs` passed into constructor to `originator`,
+        unless the event is canceled in `BeforeAssignBlindEventArgs` then key, value pair will be added automacally to `originator`.
+        @return: `True` of all key, value pairs are added; Otherwise, `False`
+        
+        Call back events are supported via `kwargs_helper.add_handler_before_assign_auto()` and `kwargs_helper.add_handler_after_assign_auto()` methods.
+        
+        Wrapper method for `KwArgsHelper.auto_assign()`
+        '''
+        self._kw_args_helper_class_instance.auto_assign()
+
+    def assign(self, key: str, field: Optional[str] = None, require: bool = False, default: Optional[object] = NO_THING, types: Optional[List[type]] = None, rules: Optional[List[Callable[[IRule], bool]]] = None) -> bool:
+        # https://docs.python.org/3/library/warnings.html#warnings.warn
+        warn("use kw_assign instead", DeprecationWarning, stacklevel=2)
+        return self.kw_assign(key=key,field=field,require=require,default=default,types=types,rules=rules)
+    
+    def kw_assign(self, key: str, field: Optional[str] = None, require: bool = False, default: Optional[object] = NO_THING, types: Optional[List[type]] = None, rules: Optional[List[Callable[[IRule], bool]]] = None) -> bool:
         '''
         Assigns attribute value to current instance passed in to constructor. Attributes automatically.
         @key: Type:str, the key of the key, value pair that is required or optional in `kwargs` passed into to constructor.
@@ -888,6 +1118,11 @@ class KwArg(HelperBase):
         return self._kw_args_helper_class_instance.assign(key=key, field=field, require=require, default=default, types=types, rules=rules)
 
     def assign_helper(self, helper: HelperArgs) -> bool:
+        # https://docs.python.org/3/library/warnings.html#warnings.warn
+        warn("use kw_assign_helper instead", DeprecationWarning, stacklevel=2)
+        return self.kw_assign_helper(helper=helper)
+
+    def kw_assign_helper(self, helper: HelperArgs) -> bool:
         '''
         Assigns attribute value using instance of HelperArgs. See `assign()` method.
         @helper: Type:HelperArgs instance to assign.
@@ -896,7 +1131,7 @@ class KwArg(HelperBase):
         self._isinstance_method(method_name='assign_helper', arg=helper,
                                 arg_name='helper', arg_type=HelperArgs, raise_error=True)
         d = helper.to_dict()
-        return self.assign(**d)
+        return self.kw_assign(**d)
 
     # region Public Methods
     def is_attribute_exist(self, attrib_name: str) -> bool:
@@ -916,7 +1151,7 @@ class KwArg(HelperBase):
             return False
         _key = attrib_name.strip()
         return hasattr(self, _key)
-    
+
     def is_key_existing(self, key: str) -> bool:
         '''
         Gets if the key exist in current kwargs.
@@ -932,6 +1167,14 @@ class KwArg(HelperBase):
         '''Get instance of KwargsHelper used to add fields current instance'''
         return self._kw_args_helper_class_instance
 
+    @property
+    def kw_unused_keys(self) -> Set[str]:
+        '''
+        Gets any unused keys passed into constructor via `**kwargs`
+
+        This would be a set of keys that were never used passed into the constructor.
+        '''
+        return self._kw_args_helper_class_instance.unused_keys
     # endregion Properties
 
-# region class KwArgs
+# endregion class KwArg
